import struct
import re
import numpy as np


def read_record(fp, typ):
    l1 = struct.unpack("<I", fp.read(4))[0]
    body = fp.read(l1)
    l2 = struct.unpack("<I", fp.read(4))[0]
    if l1 != l2:
        raise KeyError("Record is not properly close %d v %d" % (l1, l2))

    n = l1 / struct.calcsize(typ)
    body = struct.unpack("<"+typ*n, body)
    if typ == 'c':
        return ''.join(body)
    else:
        if n == 1:
            return body[0]
        else:
            return body


def guess_version(fp, inttype='i'):
    fp.seek(0)
    magic = fp.read(17).replace('\00', '')
    if magic == '\t McMule  \t':
        version = read_record(fp, 'c').strip()
        vers, intiness = re.match("v(\\d+)([NL])", version).groups()

        return int(vers), {'N': 'i', 'L': 'q'}[intiness]

    # Test v1 vs. v2
    intsize = struct.calcsize(inttype)
    fp.seek(6893 + 3*intsize)
    nrq, nrbins = read_record(fp, inttype)
    fp.seek(6925 + 5*intsize + 22*nrq + 16*nrbins*nrq)
    try:
        time = read_record(fp, 'd')
        fp.seek(0)
        return 2, inttype
    except KeyError:
        fp.seek(0)
        return 1, inttype
    except struct.error:
        fp.seek(0)
        return 1, inttype


def importvegas(filename="", fp=None, inttype='i'):
    if not fp and filename:
        fp = open(filename, 'rb')
    version, inttype = guess_version(fp, inttype)
    dic = {}

    sha = read_record(fp, 'c')
    it = read_record(fp, inttype)
    ndo = read_record(fp, inttype)
    si = read_record(fp, 'd')
    swgt = read_record(fp, 'd')
    schi = read_record(fp, 'd')
    xi = np.reshape(read_record(fp, 'd'), (17, -1))
    randy = read_record(fp, inttype)

    if version > 2:
        nrq, nrbins, namelen = read_record(fp, inttype)
    else:
        nrq, nrbins = read_record(fp, inttype)
        namelen = 6

    bounds = np.reshape(read_record(fp, 'd'), (-1, nrq))
    names = read_record(fp, 'c')
    names = [
        names[namelen*i:(i+1)*namelen].strip().replace('\00', '')
        for i in range(len(names)/namelen)
    ]
    quant = np.array(read_record(fp, 'd'))

    if version > 1:
        dic['time'] = read_record(fp, 'd')
        dic['msg'] = read_record(fp, 'c').strip().replace('\00', '')
    else:
        dic['time'] = -1
        dic['msg'] = ""

    dic['SHA'] = sha
    dic['iteration'] = it
    dic['value'] = np.array([si/swgt, np.sqrt(1/swgt)])
    if it > 1:
        dic['chi2a'] = (schi-si**2/swgt)/(it-1)
    else:
        dic['chi2a'] = -1

    if it > 1:
        if version >= 3:
            ouarray = lambda o, u, b: np.concatenate(([u], b, [o]))
        else:
            ouarray = lambda o, u, b: b

        delta = (bounds[1]-bounds[0])/nrbins
        nrbinsuo = nrbins + 2 if version > 2 else nrbins

        for i in range(nrq):
            if len(names[i]) == 0:
                continue
            x = ouarray(np.inf, -np.inf, delta[i]*(0.5+np.arange(nrbins)))
            y = quant[i:nrq*nrbinsuo:nrq] \
                / it / ouarray(1, 1, [delta[i]]*nrbins)
            e = np.sqrt(
                (quant[nrq*nrbinsuo+i::nrq]-quant[i:nrq*nrbinsuo:nrq]**2/it)
                / it / (it-1)
            ) / ouarray(1, 1, [delta[i]]*nrbins)
            dic[names[i]] = np.column_stack((x, y, e))

    if filename:
        fp.close()

    return dic
